/***************************************************************************
                          clientworld.cpp  -  description
                             -------------------
    begin                : Wed Oct 20 1999
    copyright            : (C) 1999 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/

#include "systemheaders.h"
#include "clientworld.h"
#include "obj_base.h"
#include "all.h"
#include "world_objects.h"
#include "soundsystem.h"

Clientworld::Clientworld()
{
	loader = NULL;
	ownerid = -1;	
	gorelevel = 5;
	anon_objects_inuse = 0;
}

Clientworld::Clientworld(Loader *loader_, Soundsystem *soundsystem_, int ownerid_)
{
	anon_objects_inuse = 0;
	gorelevel = 5;
	init(loader_, soundsystem_, ownerid_);
}

void Clientworld::init(Loader *loader_, Soundsystem *soundsystem_, int ownerid_)
{
	loader = loader_;
	ownerid = ownerid_;	
	World::init(loader, soundsystem_);
}

Clientworld::~Clientworld()
{
}

//-----------------------------------------------------------------------------------
void Clientworld::client_think()
{
	//process or dispatch all thoughts
	//put these into individual functions once the World class is split up into Server_world and Client_world


	//First update the state of a few lucky objects:
	while (serializer_writer.next_int() == TT_OBJECT_STATE) {
		serializer_writer.eat_int(TT_OBJECT_STATE);

		int winners_id = serializer_writer.rint();
		Object_type winners_type = (Object_type)serializer_writer.rint();

		Obj_base *update_target = get_object(winners_id,winners_type);
		if (update_target)
			update_target->serialize(&serializer_writer);
		else {
		
			//If the new object is the scoreboard, remember it for later:
			if (winners_type == ot_scoreboard) scoreboard_id = winners_id;

			//Now create the object and hand it it's data:
			Obj_base *lucky_winner = newobject(winners_type, winners_id);
			lucky_winner->serialize(&serializer_writer);
		}
	} //while (game->peek(cc_object_updates){

	//Then kill the losers
	while (serializer_writer.next_int() == TT_OBJECT_DELETE) {

		serializer_writer.eat_int(TT_OBJECT_DELETE);
		int losers_id = serializer_writer.rint();

		Obj_base *looser = get_object(losers_id,ot_error);
		if (looser)
			looser->bedone();
		else
			logmsg(lt_warning,"Recived a kiss-of-death destined for a nonexistant object (id: %i)",losers_id);
	}

	//Kill dead pixels
	while (serializer_writer.next_int() == TT_DEAD_PIXEL) {

		serializer_writer.eat_int(TT_DEAD_PIXEL);
		int x = serializer_writer.rint();
		int y = serializer_writer.rint();
		int32 pixel = x+y*bgmap_buffer.get_xsize();
		int32 savepixel=map_buffer.get_pixels()[pixel];
		map_buffer.get_pixels()[pixel]=bgmap_buffer.get_pixels()[pixel];

		Obj_particle_debris *debris = (Obj_particle_debris *)newanonobject(ot_particle_debris);
		if (debris) {
			debris->setcolor(savepixel);
			debris->init(x, y, ROUND(rnd(20,-20)), ROUND(rnd(20,-20)));
		} else
			logmsg(lt_assert,"Could not make debris");
	}
		
	//get world time
	serializer_writer.eat_int(TT_WORLD_STATE);
	settime(serializer_writer.rfloat());

	//Check for errors:
	if (!serializer_writer.empty()) {
  	logmsg(lt_error,"The serializer_writer was not empty!");
	}
	if (serializer_writer.empty_read()) {
  	logmsg(lt_error,"More bytes were requested than the writer contained!");
	}

	//let the objects do some thinking:	
	{
		World_objects::iterator i = objects->begin();
		while (i != objects->end()) {
			if (!i->second->isdone())
				i->second->client_think();
			i++;
		}
	}
	//also let the anonymous objects have a go:
	//Note: if the machine is short on cycles, this may be a good place to save a few (like: if (weak_ass_cpu) limit(anon_objects))
	int c;
	for (c=0; c<anon_objects_inuse;c++)
		if (!anon_objects[c]->isdone())
			anon_objects[c]->client_think();

	remove_dead_objects();

	//Boring array gymnastics here, I know that handling these arrays ought to be hidden away in some class somewhere,
	//but for now they are simply handled by hand, b/b+ trees may be a bit overkill though...
	//Bring out yer dead...
	{ //anon
		for (c=0; c<anon_objects_inuse;c++)
			if (anon_objects[c]->isdone()) {
					delete anon_objects[c];				
					anon_objects[c] = NULL;
			}
	
		int scribble=0;
		for (int c=0; c<anon_objects_inuse;c++)
			if (anon_objects[c])
				anon_objects[scribble++] = anon_objects[c];
		anon_objects_inuse = scribble;
	}
}


//-----------------------------------------------------------------------------------
void Clientworld::draw(Viewport *viewport)
{
	map_buffer.draw_copy(viewport,0,0,viewport->xpos,viewport->ypos,viewport->get_xsize(),viewport->get_ysize());

	for (int c=0; c<anon_objects_inuse;c++) {
		if (!anon_objects[c]->isdone())
			anon_objects[c]->draw(viewport);
	}

	World_objects::iterator i = objects->begin();
	while (i != objects->end()) {
		if (!i->second->isdone())
			i->second->draw(viewport);

		i++;
	}
/*	
	for (int c=0; c<real_objects_inuse;c++)
		if (!real_objects[c]->isdone())
			real_objects[c]->draw(viewport);
*/
}


//-----------------------------------------------------------------------------------
Obj_base *Clientworld::newanonobject(Object_type objecttype)
{
	if (anon_objects_inuse+1 >= MAX_ANON_OBJECTS) {
		logmsg(lt_error,"There are now %i anonymous objects in the world, not going to create a %s",anon_objects_inuse,OBJECT_NAMES[objecttype]);
  	return NULL;
	}

	Obj_base *newobject = NULL;
	switch (objecttype) {
		case ot_weapon_test	:
		case ot_weapon_minigun	:
		case ot_weapon_shotgun	:
		case ot_weapon_megashotgun	:
		case ot_weapon_railgun	:
		case ot_weapon_plasmagun	:
		case ot_weapon_larpa    :
		case ot_weapon_rocket    :
		case ot_weapon_mininuke	:
		case ot_weapon_bignuke	:
		case ot_weapon_clusterbomb	:
		case ot_bignuke	:
		case ot_rocket  :
		case ot_grenade  :
		case ot_clusterbomb  :
		case ot_plasmaball :
		case ot_railslug	:
		case ot_hook :
		case ot_scoreboard:
		case ot_error :
		case ot_tinygranade :
		case ot_testobject	:
		case ot_explosion		:
		case ot_player_avatar :
		case ot_player_data :
		case ot_projectile:
		case ot_gib :
			logmsg(lt_assert,"Someone tried to create an non-anonymous with newanonobject()");
			return NULL; //these are not anon objects

		case ot_particle_smoke:  newobject = new Obj_particle_smoke(this, -1); break;
		case ot_particle_blood:  newobject = new Obj_particle_blood(this, -1); break;
		case ot_particle_debris: newobject = new Obj_particle_debris(this, -1); break;
		case ot_particle_railtrail: newobject = new Obj_particle_railtrail(this, -1); break;
		case ot_particle_spark: newobject = new Obj_particle_spark(this, -1); break;

		//case ot_dust
		//case ot_smoke
		//case ot_particle
		
	}

	anon_objects[anon_objects_inuse++] = newobject;

	return newobject;
}

//-----------------------------------------------------------------------------------
void Clientworld::makehole(int x, int y, int diameter)
{
	if (diameter > 16) diameter = 16;
	if (diameter < 1) diameter = 1;
	
	if (x < 1) x = 1;
	if (x >= map_buffer.get_xsize()-1) x = map_buffer.get_xsize()-2;
	if (y < 1) y = 1;
	if (y >= map_buffer.get_ysize()-1) y = map_buffer.get_ysize()-2;

	//find a couple of colors to give to the debris	
	int32 pcolors[16];
	int pcolors_inuse = 0;
	int offset = x + y*map_buffer.get_xsize();
	for (int yc=-1;yc <= 1;yc++)
		for (int xc=-1;xc <= 1;xc++) {
			int32 pix = map_buffer.get_pixels()[offset+xc+yc*map_buffer.get_xsize()];
			if (IS_EARTH(pix) && pcolors_inuse < 16)
	  		pcolors[pcolors_inuse++] = pix;
		}

	//become desperate and take every color in sight if we can't find enough at the center.
	if (pcolors_inuse < 1){
		for (int yc=-5;yc <= 5;yc++)
			for (int xc=-5;xc <= 5;xc++) {
				int32 pix = map_buffer.get_pixels()[offset+xc+yc*map_buffer.get_xsize()];
				if (IS_EARTH(pix) && pcolors_inuse < 5)
		  		pcolors[pcolors_inuse++] = pix;
			}
	}	

	//do the actual hole putting
	holes[diameter-1]->draw_map(&map_buffer,&bgmap_buffer,x,y);

	//spawn the debris:
	if (pcolors_inuse) {
		int radius = diameter/2;
		for (int c=0; c <radius;c++) {
 	  	Obj_particle_debris *debris = (Obj_particle_debris *)newanonobject(ot_particle_debris);

			if (debris) {
				debris->setcolor(pcolors[rnd(0,pcolors_inuse-1)]);
				debris->init(x+rnd(-2,2), y+rnd(-2,2), ROUND(rnd(20,-20)),	ROUND(rnd(20,-20)));
			}
		}
	}
}

bool Clientworld::createmap(Net_joinreply &joinreply, char *map_data)
{
	World::createmap(joinreply.map_xsize,joinreply.map_ysize,joinreply.randomseed);

	logmsg(lt_debug,"Restoring map state from a %i byte diff",joinreply.compressed_size);

	unsigned int pixelcount = joinreply.map_xsize*joinreply.map_ysize;
	unsigned int pixelbytes = pixelcount*sizeof(int32);

	//unpack the map_data into the patch.
 	int32 *diff = (int32*)malloc(pixelbytes);

	
	if (joinreply.zcompressed) {
		unsigned long uncompressed_bytes = pixelbytes;
		int zerror = uncompress(
				(unsigned char *)diff,
				&uncompressed_bytes,
      	(const unsigned char*)map_data,
				joinreply.compressed_size
			);

		if (zerror == Z_OK) {
    	logmsg(lt_debug,"uncompressed the diff alright");
		}else{
    	logmsg(lt_error,"Unable to uncompress the diff! the error was:%i",zerror);
		}

	} else {
		memcpy(diff,map_data,pixelbytes);
	}

	//apply the patch to the map
	for (unsigned int i=0;i<pixelcount;i++) {
		int32 op = map_buffer.get_pixels()[i]; //current pixel
		int32 dp = diff[i];

		if (!IS_PASSABLE(op) && dp == PF_TYPE_HOLE) {
			map_buffer.get_pixels()[i] = bgmap_buffer.get_pixels()[i];
		} else
			map_buffer.get_pixels()[i] = op ^ dp;
	}
	//memcpy(map_buffer.pixels,map_data,joinreply.compressed_size);

	free(diff);

	return true;
}














































