/***************************************************************************
                          controls.h  -  description
                             -------------------
    begin                : Sat Oct 30 1999
    copyright            : (C) 1999 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/


#ifndef CONTROLS_H
#define CONTROLS_H

#include "fluff.h"
#include "systemheaders.h"
#include "nil_math.h"
#include "serializable.h"

/**
  *@author Flemming Frandsen
  */


//this class is used simply to collect all the control information that must be sent to the server.
class Controls : public Serializable {
public:

	//control flags:
	typedef enum {

	/*
		This is all there is control wise, it's pretty effective,
		but it also means that there can only be 16 static and 16 impulse controls
	*/

	//static flags:
		cf_walk						= 0x00000001,
		cf_shoot					= 0x00000002,
		cf_hook_in				= 0x00000004,
		cf_hook_out				= 0x00000008,
		cf_direction_east	=	0x00000010,
  	cf_stat_aux1			= 0x00000020,
		cf_stat_aux2			= 0x00000040,
		cf_stat_aux3			= 0x00000080,
		cf_stat_aux4			= 0x00000100,
		cf_stat_aux5			= 0x00000200,
		cf_stat_aux6			= 0x00000400,
		cf_stat_aux7			= 0x00000800,
		cf_stat_aux8			= 0x00001000,
		cf_stat_aux9			= 0x00002000,
		cf_stat_aux10			= 0x00004000,
		cf_stat_aux11			= 0x00008000,

	//impulse flags
		cf_jump						= 0x00010000,
		cf_dig						=	0x00020000,
		cf_hook_fire			= 0x00040000,
		cf_hook_release		= 0x00080000,
		cf_die_horribly		= 0x00100000,
		cf_imp_aux1				= 0x00200000,
		cf_imp_aux2				= 0x00400000,
		cf_imp_aux3				= 0x00800000,
		cf_imp_aux4				= 0x01000000,
		cf_imp_aux5				= 0x02000000,
  	cf_imp_aux6				= 0x04000000,
		cf_imp_aux7				= 0x08000000,
		cf_imp_aux8				= 0x10000000,
		cf_imp_aux9				= 0x20000000,
		cf_imp_aux10			= 0x40000000,
		cf_imp_aux11			= 0x80000000,

	//special:
		cf_not_impulse		= 0x0000FFFF

	} Controlflags;


	Controls()													{elevation = MINELEVATION*2; controlflags = 0;}

	//bit diddeling
	void set_flag(int flag, bool state=true) {
		if (state)
			controlflags |= flag;
		else
			controlflags &= ~flag;
	}
	void clear_flag(int flag) 					{controlflags &= ~flag;}
	bool get_flag(int flag) 						{return (controlflags & flag) != 0;}

	//elevation
	float get_elevation() 							{return elevation;}
	void set_elevation(float elev) 			
		{
			elevation = elev;
			if (elevation < MINELEVATION) elevation = MINELEVATION;
			if (elevation > MAXELEVATION) elevation = MAXELEVATION;
		}	

	int get_weapon() 										{return weapon;}
	void set_weapon(int weapon_) 				{weapon = weapon_;}

	void clear_impulses() 							{controlflags &= cf_not_impulse;}

	//client/server	interface:
	void serialize(Serializer *serializer)
		{
    	serializer->rw(controlflags);
    	serializer->rw(elevation);
			serializer->rw(weapon);

			//if we are setting the state of the controls object do some extra checking
			if (!serializer->isreading()) {
				if (elevation < MINELEVATION) elevation = MINELEVATION;
				if (elevation > MAXELEVATION) elevation = MAXELEVATION;
			}
		}

	void print(char *msg)
		{
			logmsg(lt_debug,"%s * controlflags: %x, elevation: %.2f, weapon: %i",msg,controlflags,elevation,weapon);
		}

protected:

	float elevation;
	int weapon;
	int controlflags;
};

#endif












