/***************************************************************************
                          graphic_bar.cpp  -  description
                             -------------------
    begin                : Sat Dec 25 1999
    copyright            : (C) 1999 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/


#include "graphic_bar.h"
#include "fluff.h"
#include "string.h"
#include "nil_math.h"

void Graphic_bar::setgeom(int xpos_, int ypos_, int xsize_, int ysize_)
{
	if (xsize != xsize_ || ysize != ysize_)
		create(xsize_, ysize_);
	xpos = xpos_;
	ypos = ypos_;
}

void Graphic_bar::draw(Raw_surface *raw_target)
{
	int target_x = xpos + (xsize >> 1);
	int target_y = ypos + (ysize >> 1);
	draw_view(raw_target, target_x, target_y);
}

Graphic_bar::Graphic_bar() : Graphic()
{
	bar_color = PIXELA(255,0,0,6);
	border_color = PIXELA(255,255,255,15);
	bar_length = 5;
	setpos(0.5);
	xpos = 0;
	ypos = 0;
}

Graphic_bar::~Graphic_bar()
{
}

//Note: consolidate ALL rendering gadgets in the rawbuffer class
//like line and horline and stuff
inline void horline(int32 *pixels, int length, int32 color)
{
	for(int x=0;x<length;x++) *(pixels+x) = color;
}

void Graphic_bar::setpos(float pos)
{
	if (pos < 0 || pos > 1) {
  	logmsg(lt_error,"Invalid pos value passed to Graphic_bar::setpos(): %.3f",pos);
		return;
	}

	int new_bar_length = ROUND((xsize-2)*pos);

	if (new_bar_length != bar_length) {
		bar_length = new_bar_length;
		render();
	}
}

void Graphic_bar::render()
{
	if (xsize <= 0 || ysize <= 0 || !pixels) return;
	
	//Start by clearing the area:
  memset(pixels,0,xsize*ysize*sizeof(int32));

  //First make the border
	draw_rectangle_border_raw(0,0,xsize-1,ysize-1,border_color);

	//Draw the bar
	draw_rectangle_raw(1,1,bar_length+1,ysize-2+1,bar_color);

	//Draw the top of the bar	
	draw_vertical_line_raw(bar_length+1,1,xsize-1,border_color);
}






