/***************************************************************************
                          nil_main.cpp  -  description
                             -------------------
    begin                : Mon Nov 15 1999
    copyright            : (C) 1999 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/

#include "systemheaders.h"
#include "raw_surface.h"
#include "nil_server.h"
#include "nil_client.h"
#include "fluff.h"
#include "configuration.h"

#include "loader.h"
#include <vector>

#ifdef _WIN32
	#define NILRC ".nilrc"
#else
	#define NILRC "~/.nilrc"
#endif

#define NILRC_LAST NILRC ".last_good"

int main(int argc,char **argv)
{
	setmsgsource("NiL"); //The string that appears before the log messages.

	//check a few things:
	if (strcmp(OBJECT_NAMES[ot_error],"ERROR")) {
  	logmsg(lt_fatal,"You have fucked up, OBJECT_NAMES[ot_error] was not ERROR, it was: \"%s\" take a look at fluff.h",OBJECT_NAMES[ot_error]);
		return 1;
	}

	fprintf(stdout,"NiL - Worms, Violence and the GPL\n\n");
	Configuration config;
	
	if (!config.load_file(NILRC)) {
		fprintf(stdout,"Warning: Unable to read " NILRC ":\n%s\n",config.get_error());
	}

	if (!config.parse_cmdline(argc,argv)) {
  	fprintf(stdout,"Unable to understand commandline: %s\n",config.get_error());
		fprintf(stdout,"\nUsage: nil [<option>=<value>]\n");
		fprintf(stdout,"Options can be given in " NILRC " configuration file or\n");
		fprintf(stdout,"overridden from the command line ('nil mode=server').\n\n");
	
  	fprintf(stdout,"The options I understand (on commandline or in ~/.nilrc) are:\n");
		config.print_manual(stdout);
		return 1;
	}

	if (!config.check_options()) {
		fprintf(stdout,"A value failed the secondary check:\n%s\n",config.get_error());
		return 2;
	}

	config.save_options(NILRC_LAST);

	setchattiness((Logtype)config.get_option_int("chattiness"));

	const char *mode = config.get_option("mode");

	if (strcasecmp(mode,"server") == 0) {
		run_server(config);

  } else if (strcasecmp(mode,"client") == 0) {
		run_client(config);

	} else if (strcasecmp(mode,"serverclient") == 0) {
#ifdef _WIN32
		std::vector<const char*> args(argv,argv+argc);
		args.push_back("mode=client");
		args.push_back(NULL);

		int pi = _spawnv(_P_NOWAIT, argv[0], &args[0]);
		run_server(config);
		TerminateProcess((HANDLE)pi, 0);
#else
		logmsg(lt_message,"Going to fork off a client and run a server...");
		//fork off a client before running the server
		pid_t pid = fork();
		if (pid == 0)
			run_client(config);
		else {
			run_server(config);
			kill(pid,SIGTERM);
		}
#endif
	} else if (strcasecmp(mode,"clientserver") == 0) {
#ifdef _WIN32
		std::vector<const char*> args(argv,argv+argc);
		args.push_back("mode=server");
		args.push_back(NULL);

		int pi = _spawnv(_P_NOWAIT, argv[0], &args[0]);
		run_client(config);
		TerminateProcess((HANDLE)pi, 0);
#else
		logmsg(lt_message,"Going to fork off a server and run a client...");
		//fork off a server before running the client
		pid_t pid = fork();
		if (pid == 0)
			run_server(config);
	 	else {
			run_client(config);
			kill(pid,SIGTERM);
		}
#endif
	} else if (strcasecmp(mode,"cleanup") == 0) {
		logmsg(lt_debug,"This mode is obsolete");
	} else if (strcasecmp(mode,"savelump") == 0) {

		//expand the output filename:
		char expanded_filename[200];
		if (!expand_path(config.get_option("output_lump"),expanded_filename,sizeof(expanded_filename))) {
  		logmsg(lt_fatal,"Unable to expand the path %s",config.get_option("output_lump"));
			return false;
		}

		Loader loader;
		loader.addsources(config.get_option("data"));

		//save a lump:
		int bs = 10*1000*1000;
		char *buffer = (char*)malloc(bs);
		int lump_size= loader.save_lump(buffer,bs);
		if (lump_size > 0) {
			logmsg(lt_debug,"Saving %i bytes to %s",lump_size,expanded_filename);
      FILE *file = fopen(expanded_filename,"w");
			fwrite(buffer,lump_size,1,file);
			fclose(file);
		}
		free(buffer);

	} else {
  	logmsg(lt_fatal,"This cannot possibly happen, mode was:%s",mode);
		return 3;
	}

	return 0;
}
