/***************************************************************************
                          nil_server.cpp  -  description
                             -------------------
    begin                : Sun Nov 14 1999
    copyright            : (C) 1999 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/

#include "systemheaders.h"
#include "nil_server.h"
#include "fluff.h"
#include "serverworld.h"
#include "loader.h"
#include "tcp_server.h"
#include "soundsystem.h"

void run_server(Configuration &config)
{
	setmsgsource("S");
	logmsg(lt_message,"Running server");

	Tcp_server tcp_server;

	tcp_server.set_fudge(
		config.get_option_int("net_normal_backlog"),
		config.get_option_int("net_abnormal_backlog"),
		config.get_option_int("net_max_wait_time")
	);
	
	tcp_server.bind_and_listen(config.get_option_int("server_port"),config.get_option_int("net_multithread"));

	Loader loader;
	loader.addsources(config.get_option("data"));

	Soundsystem soundsystem(&loader);

	//set up the server:
	Serverworld serverworld(&loader,&soundsystem);
	tcp_server.serverworld = &serverworld;

	Config_xy world_size = config.get_option_xy("world_size");
	serverworld.createmap(world_size.x,world_size.y,config.get_option_int("randomseed"));
	serverworld.save_original_map();
	logmsg(lt_debug,"map created");
	
	int traffic = 0;
	int frames = 0;
	bool done = false;

	unsigned int cur_time = time_ms();
	float gametime = 0.0;
	float stat_start = 0.0;
	unsigned int last_time = cur_time;
	int recycle_counter = 0;

  while (!done) {

		if (request_quit)
			done = true;

		{ //find the new time
			last_time = cur_time;
			cur_time = time_ms();
			unsigned int delta_time = time_elapsed(last_time,cur_time);
		
			gametime += delta_time/1000.0;
		}

		//see if it is time to recycle serializers
		if (recycle_counter-- <= 0) {
			tcp_server.recycle_serializers();
			//do a recycle run every 5 frames (maybe this shold be every frame, experiment)
			recycle_counter = 5;
		}

		serverworld.settime(gametime);
		serverworld.server_think(&tcp_server.update_queue);

		//update clients, update their controls and let new clients join.
		tcp_server.poll();

		//do some stats:
		traffic += serverworld.last_reader_length;
		frames++;
    //print stats:
		if (frames >= 150) {
			logmsg(lt_message,"Bytes/sec: %.1f, fps: %.2f, dt: %.2f, time:%.2f",
				traffic/(gametime-stat_start),
				frames/(gametime-stat_start),
				gametime-stat_start,
				gametime
			);
    	traffic = 0;
			frames = 0;
			stat_start = gametime;
		}

		//delay one tick
		while (time_elapsed(cur_time,time_ms()) < 20)
			usleep(1000);

  }

 	return;
}