/***************************************************************************
                          obj_hook.cpp  -  description
                             -------------------
    begin                : Thu Dec 23 1999
    copyright            : (C) 1999 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/


#include "obj_hook.h"
#include "all.h"
#include "fluff.h"

//-------------------------------------------------------
Obj_hook::Obj_hook(World *world_, int id) : Obj_base(world_,id)
{
	air_resistance = DEFAULT_DRAG / 5.0;
	old_age = age();
	rope_length = 25;
}

//-------------------------------------------------------
Obj_hook::~Obj_hook()
{
}

//-------------------------------------------------------
void Obj_hook::serialize(Serializer *serializer)
{
	Obj_base::serialize(serializer);
	serializer->rw(stuck);
	serializer->rw(avatar_id);
	serializer->rw(rope_length);
}


//-------------------------------------------------------
void Obj_hook::server_think()
{
//  Serverworld *serverworld = (Serverworld *)world;
	Obj_player_avatar *avatar = (Obj_player_avatar *)world->get_object(avatar_id,ot_player_avatar);
	Vector av;

	float bungee_length = rope_length;

	if (avatar) {
		float ax,ay;
		avatar->getpos(ax,ay);
  	av.x = xpos-ax;
  	av.y = ypos-ay;

		float len = av.length();
		av.normalize();
		av.scale(age()-old_age);
		
   	av.scale(2*DEFAULT_ACCEL*len/bungee_length); //FLEM av.scale(DEFAULT_ACCEL*len/bungee_length);

	} else {
		bedone();
		return;
	}

	if (stuck) {
		//check to see if the hook has come loose
		if (is_passable(ROUND(xpos),ROUND(ypos))) {
			yaccel = DEFAULT_ACCEL;
      stuck = false;
			setdirty();
		}

		//Pull on the owner avatar
    if (avatar) {
    	avatar->push(av);
		}
	} else {
  	//pull towards owner a little bit
    av.scale(-0.1);
		push(av);
		
    //see if we have hit something:
		Map_hit hit = update_simple(); //this calls client_think() for us and checks for collisions

		if (hit.hittype != ht_none) {
  		stuck = true;

			Vector speed;
			speed.x = old_xspeed;
			speed.y = old_yspeed;
			speed.normalize();
		  speed.scale(4); //FLEM speed.scale(2);

			xpos = hit.xc+ROUND(speed.x);
			ypos = hit.yc+ROUND(speed.y);

			xspeed = 0;
			yspeed = 0;
			yaccel = 0; //dont fall.

			setdirty();
		}
	}
	old_age = age();
}

//-------------------------------------------------------
void Obj_hook::draw(Viewport *viewport)
{
	int x = ROUND(xpos)-viewport->xpos;
	int y = ROUND(ypos)-viewport->ypos;

	//Draw the rope:
	Obj_player_avatar *avatar = (Obj_player_avatar *)world->get_object(avatar_id,ot_player_avatar);
	if (avatar) {
		float ax,ay;
		avatar->getpos(ax,ay);
		viewport->draw_line(ROUND(xpos)-viewport->xpos,ROUND(ypos)-viewport->ypos,ROUND(ax)-viewport->xpos,ROUND(ay-4)-viewport->ypos,PIXEL(150,128,100));
	}

	//Draw the hook:
	if (x >= 0 && y >= 0 && x < viewport->get_xsize() && y < viewport->get_ysize()) {
//    *(viewport->get_pixels()+x+y*viewport->get_xsize()) = PIXEL(255,255,0);
		viewport->draw_pixel(x,y,PIXEL(255,255,0));
	}
}

void Obj_hook::init(int owner,float xpos_, float ypos_, float xspeed_, float yspeed_)
{
	ownerid = owner;
	xpos = xpos_;
	ypos = ypos_;
	xspeed = xspeed_;
	yspeed = yspeed_;
	stuck = false;	
	Obj_player_avatar *avatar = get_owner_avatar();
	if (avatar) avatar_id = avatar->getid();

	setdirty();
}

void Obj_hook::reel_out()
{
	rope_length += 2 + rope_length/20;
	if (rope_length > 500)
		rope_length = 500;
	setdirty();
	logmsg(lt_debug,"rope: %.0f",rope_length);
}

void Obj_hook::reel_in()
{
	rope_length -= 2 + rope_length/20;
	if (rope_length < 1)
		rope_length = 1;

	setdirty();
	logmsg(lt_debug,"rope: %.0f",rope_length);
}
