/***************************************************************************
                          obj_rocket.cpp  -  description
                             -------------------
    begin                : Tue Feb 22 2000
    copyright            : (C) 2000 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/


#include "obj_rocket.h"
#include "all.h"

Obj_rocket::Obj_rocket(World *world_, int id) : Obj_base(world_,id)
{
	for(int i=0; i<8; i++)
		sprite[i]=0;
}
Obj_rocket::~Obj_rocket()
{
}
       
void Obj_rocket::init(int owner, float xpos_, float ypos_, float xspeed_, float yspeed_, float trail_, float power_, int size_)
{
	ownerid = owner;
	xpos = xpos_;
	ypos = ypos_;
	xspeed = xspeed_;
	yspeed = yspeed_;
	yaccel = DEFAULT_ACCEL/3;
	createtime=world->gettime();
	trail=trail_;
	power=power_;
	size=size_;
	last=0;
	air_resistance=0.007;
}

//The frames are numbered like this:
//234
//1 5
//076
void Obj_rocket::draw(Viewport *viewport)
{
	// *WARNING* qute: When I shoot up the down gfx is show. Damn.
	// *WARNING* ff: WTF?

	int x = ROUND(xpos)-viewport->xpos;
	int y = ROUND(ypos)-viewport->ypos;
	float rad=atan(yspeed/xspeed); //Calculated the dir.
	rad=2*PI-rad; //Flip it.
	rad=PI-rad;   //Rotate it.
//It now starts at 1,0 with PI/2 in 0,-1) PI in -1,0 and 3*PI/2 in 0,1.

	if(rad<0) rad+=2*PI; //Add it up. In math it doesn't matter, but here it does.
	int dir=0;
	if(xspeed>0) {
		dir=1;
		rad=rad+PI;
	} //tried this and it worked. Don't know why.
	if(rad>2*PI) rad-=2*PI; //Add it up. In math it doesn't matter, but here it does.
	
	int frame=0;
	if(rad<PI/16)
		frame=5;
	else if(rad<(PI/2-PI/16))
		frame=6;
	else if(rad<(PI/2+PI/16))
		frame=7;
	else if(rad<(PI-PI/16))
		frame=0;
	else if(rad<(PI+PI/16))
		frame=1;
	else if(rad<(3*PI/2-PI/16))
		frame=2;
	else if(rad<(3*PI/2+PI/16))
		frame=3;
	else if(rad<(2*PI-PI/16))
		frame=4;
	else {
		logmsg(lt_assert,"void Obj_rocket::draw(Viewport *viewport) is buggy!");
		frame=5;
	}
		
	if (!sprite[frame]) {
		// *WARNING* Could this be static??  Same goes for grep STATIC_STRING. email qute@cutey.com if you know the answer
		char s[100];
		sprintf(s,"obj/larpa/larpa%i.rgbf",frame);
		logmsg(lt_fatal,"fil: %s rad=%f\n  dir %d",s,rad,dir);
		sprite[frame] = world->loader->graphics.getgraphic(s);
		if (!sprite[frame]) return;
	}
	
	sprite[frame]->draw_view(viewport,x,y);
}

void Obj_rocket::serialize(Serializer *serializer)
{
	Obj_base::serialize(serializer);
       
	serializer->rw(trail);
	serializer->rw(power);
	serializer->rw(last);
	serializer->rw(size);
}

void Obj_rocket::server_think()
{
	Serverworld *serverworld = (Serverworld *)world;
       
	Map_hit hit = update_simple(true); //this calls client_think() for us and checks for collisions

	bool blow=false;

	if(trail>0)
		if((age()>0.03) && (age()>last+trail)) {
			last+=trail;
			Obj_projectile *bullet = (Obj_projectile *)serverworld->newobject(ot_projectile);
			if (bullet) bullet->init(
				ownerid,
				xpos,ypos,
				xspeed/10+rnd(-15,15),
				25+yspeed/5+rnd(-1,1),
				100,
				4,
				0,
				0
			);
		}

	if (hit.hittype != ht_none) {
		net_dirty = true;
		blow=true;
	}
       
	if(blow) {
		done=true;
		Obj_explosion *explosion = (Obj_explosion *)serverworld->newobject(ot_explosion);

		if (explosion) explosion->init(
			ownerid,
			xpos,ypos,
			size,
			power
		);

	}
}

