/***************************************************************************
                          obj_weapon.cpp  -  description
                             -------------------
    begin                : Wed Dec 15 1999
    copyright            : (C) 1999 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/


 #include "obj_weapon.h"
#include "all.h"

Obj_weapon::Obj_weapon(World *world_, int id) : Obj_base(world_,id)
{
	readyness = 1;
	reloading = false;

	//Set some sort of defaults, ALL weapons must set their own values
	reload_speed = 0.1;
	shot_cost = 0.5;
	reshoot_time = 0.5;
	last_shot_time = 0;
	old_age = age();
}

Obj_weapon::~Obj_weapon()
{
}

void Obj_weapon::init(int ownerid_)
{
	ownerid = ownerid_;
	readyness = 0;
	reloading = false;
	curtime = world->gettime();
	old_age = age();
}

void Obj_weapon::serialize(Serializer *serializer)
{
	serializer->rw(ownerid);
	serializer->rw(readyness);
	serializer->rw(reloading);
}

bool Obj_weapon::fire(float x, float y, float aim_angle)
{
	if (world->isclient()) return false;
  if (!canfire()) return false;
	if (reshoot_time > age()-last_shot_time) return false;

	Obj_player_avatar *avatar = get_owner_avatar();
	if (avatar)	avatar->getspeed(xspeed,yspeed);	
	
	Vector v = rad2vector(aim_angle);
	knock_back_direction.zero();
 	dofire(ROUND(x)+v.x*5,ROUND(y-4)+v.y*5,v,aim_angle);	
	readyness -= shot_cost;
	last_shot_time = age();

	//tell the client about the new decrease in readyness
  setdirty();

	//apply the knock back
	knock_back_direction.normalize();
	knock_back_direction.scale(-knock_back);
	if (avatar)
		avatar->push(knock_back_direction);
	else
		logmsg(lt_debug,"Unable to locate the avatar for knockback");		
	
	return true;
}

void Obj_weapon::server_think()
{
	client_think();

	//see if we should reload:
	if (!reloading && readyness < shot_cost) {
 		reloading = true; 	                	
		setdirty();
		logmsg(lt_debug,"Reloading %s",getname());
		old_age = age();
	}

	//Reload the gun:
	if (reloading) {
		//calc time since last update:
		float delta_time = age()-old_age;

		readyness += reload_speed * delta_time;

		//see if we should stop reloading
		if (readyness >= 1) {
    	readyness = 1;
			reloading = false;
			logmsg(lt_debug,"Done reloading %s",getname());
		}
		setdirty();
	}
	old_age = age();
}

bool Obj_weapon::verify()
{
	return !((readyness < -0.1) || (readyness > 1.1));
}



















