/***************************************************************************
                          serializer_writer.cpp  -  description
                             -------------------
    begin                : Sat Jan 15 2000
    copyright            : (C) 2000 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/


#include "serializer_writer.h"
#include "systemheaders.h"
#include "fluff.h"

//-----------------------------------------------------------
Serializer_writer::Serializer_writer()
{
	buffer_size = 1000;
	buffer_inuse = 0;
	buffer = (char *)malloc(buffer_size);
	empty_read_error = false;
	curmsg = 0;
}

//-----------------------------------------------------------
Serializer_writer::~Serializer_writer()
{
	free(buffer);
}

//-----------------------------------------------------------
char *Serializer_writer::setbuffer(int newlength)
{
	if (newlength > buffer_size) {
		buffer_size=newlength;
  	buffer = 	(char *)malloc(buffer_size);
	}
	
	buffer_inuse = newlength;
	rewind();
	return buffer;
}

//-----------------------------------------------------------
int Serializer_writer::next_int()
{
	if (empty() || curmsg+(signed)sizeof(int) > buffer_inuse)
		return -1;
	else
		return *((int*)(&buffer[curmsg]));
}

//-----------------------------------------------------------
bool Serializer_writer::eat_int(int expected)
{
	int i = rint();
	if (i == expected)
		return true;
	else {
	 	logmsg(lt_error,"eat_int(%i) found %i instead",expected,i);
		return false;
	}
}

//-----------------------------------------------------------
inline bool Serializer_writer::get_bytes(char *data, int length)
{
	if (curmsg+length > buffer_inuse) {
		logmsg(lt_error,"tried to read %i bytes from Serializer_writer, that was too many",length);
		empty_read_error = true;
		return false;
	} else {
		memcpy(data, buffer+curmsg, length);
		curmsg += length;
		return true;
	}
}

//-----------------------------------------------------------
void Serializer_writer::rw(char &value)		
{
	get_bytes((char *)&value,sizeof(value));
}

//-----------------------------------------------------------
void Serializer_writer::rw(int &value)
{
	get_bytes((char *)&value,sizeof(value));
}

//-----------------------------------------------------------
void Serializer_writer::rw(int32 &value)
{
	get_bytes((char *)&value,sizeof(value));
}

//-----------------------------------------------------------
void Serializer_writer::rw(bool &value)
{
	get_bytes((char *)&value,sizeof(value));
}

//-----------------------------------------------------------
void Serializer_writer::rw(float &value)
{
	get_bytes((char *)&value,sizeof(value));
}

//-----------------------------------------------------------
void Serializer_writer::rw(double &value)
{
	get_bytes((char *)&value,sizeof(value));
}

//-----------------------------------------------------------
void Serializer_writer::rwstr(char *value, int maxsize)
{
	int len=min(rint(),maxsize-1);
	get_bytes(value,len);
	value[len]=0;
}

//-----------------------------------------------------------
void Serializer_writer::rwblob(char *blob, int size)
{
	get_bytes(blob,size);
}
