/***************************************************************************
                          update_queue.h  -  description
                             -------------------
    begin                : Sun Feb 13 2000
    copyright            : (C) 2000 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/
#ifndef UPDATE_QUEUE_H
#define UPDATE_QUEUE_H
#include "systemheaders.h"

/*
	This encapsulates the queue of updates going to the clients.
	The reason for recycling the Serializers is that it is vastly
	faster (because the serializers have quite a large internal
	buffer that grows to fit the kinds of traffic that the
  serializer sees)
*/
class Update_queue_recycler;
class Update_queue_serializers;
class Serializer_reader;
class Update_queue {
public:
	Update_queue();
	~Update_queue();

	//Returns a used serializer or a new one, if there is no serializer in the recycle bin
	Serializer_reader *new_reader();

	//This hands a serializer full of data to the queue
	void submit_update(Serializer_reader *newupdate);

	//recycle all the serializers older than this id to the recycler
	void recycle(int update_id);

 //accessed from the client connection threads:
	//Returns the the id of the latest update
	int get_latest_update_id() {return latest_update_id;}
	
	//Returns the update with the given id
	Serializer_reader *get_update(int update_id);

protected:
	int latest_update_id;
	Update_queue_recycler *recycler;
	Update_queue_serializers *serializers;
	pthread_mutex_t queue_lock;
};

#endif

